@@ -8,14 +8,15 @@ if (!checkMongoLocal()) exit_file("Reason: no local MongoDB")
 if (!checkInternet())   exit_file("Reason: no internet connectivity")
 if (!checkBinaries())   exit_file("Reason: no binaries php or sed or perl")
 
-# create database object
-dbc <- nodbi::src_mongo(
-  db = mongoLocalRwDb,
-  collection = mongoLocalRwCollection,
-  url = "mongodb://localhost")
-
 #### CTGOV ####
 tf <- function() {
+
+  # create database object
+  dbc <- nodbi::src_mongo(
+    db = mongoLocalRwDb,
+    collection = mongoLocalRwCollection,
+    url = "mongodb://localhost")
+
   # register clean-up
   on.exit(expr = {
     try({
@@ -24,6 +25,13 @@ tf <- function() {
     },
     silent = TRUE)
   })
+
+  # check server
+  if (httr::status_code(
+    httr::HEAD("https://clinicaltrials.gov/ct2/search",
+               httr::timeout(10L))) != 200L
+  ) return(exit_file("Reason: CTGOV not working"))
+
   # do tests
   source("ctrdata_ctgov.R", local = TRUE)
 }
